// Real.h: header file for real number functions and constants.
//
//////////////////////////////////////////////////////////////////////

#ifndef REAL_H
#define REAL_H

/** This definition helps treat all application dependent
* numbers in a uniform manner.
* Currently it is implemented as a double percision floating-point
* C type.
*/
typedef float real;

//////////////////////////////////////////////////////////////////////
// Real number constants
//////////////////////////////////////////////////////////////////////

extern const real REAL_MAX;
extern const real REAL_MIN;
extern const real EPSILON;
extern const real PI;
extern const real TWO_PI;

//////////////////////////////////////////////////////////////////////
// Real number functions
//////////////////////////////////////////////////////////////////////

/** Returns the absolute value of a difference between real numbers.
* return: the absolute value of the difference : | lhs-rhs |.
* param lhs: a real number.
* param rhs: a real number.
*/
inline real AbsDiff(real lhs, real rhs)
{
	real diff = lhs - rhs;
	return diff >= 0 ? diff : -diff;
}

/** Compares two real numbers.
* This function is necessary, since the c operator== is
* very sensitive to small variations, and two different paths 
* to the same input numbers won't always lead to the same answer.
* This means, for example, that floating-point numbers 
* which "should be" equal are not "always" equal.
* return: true if the two numbers are closer than EPSILON.
* param lhs: a real number used in the comparison. 
* param rhs: a real number used in the comparison.
*/
inline bool REAL_EQUAL(real lhs, real rhs)
{
	return AbsDiff(lhs, rhs) <= EPSILON;
}

// x is smaller than y
inline bool REAL_LESS(real x, real y)
{
 if (y - x > EPSILON)
 {
   return true;
 }
 return false;
}

// x is greater than y
inline bool REAL_GREATER(real x, real y)
{
 if (x - y > EPSILON)
 {
   return true;
 }
 return false;
}

// x is less than or equal to y
inline bool REAL_LESS_OR_EQUAL(real x, real y)
{
	return !REAL_GREATER(x, y);
}

// x is greater than or equal to y
inline bool REAL_GREATER_OR_EQUAL(real x, real y)
{
 return !REAL_LESS(x,y);
}

inline real RealAbs(real x)
{ return ( (x > 0)? x : (-x) ); }


inline real RealSquare(real x)
{ return x*x; }

#endif // REAL_H