#pragma warning( disable : 4786 )
#pragma warning( disable : 4503 )

#include <string.h>
#include <GL/glut.h>
#include <glui.h>

#include "OpenGLtexture.h"		// for texture handling
#include "TexturedMesh.h"
#include "ColorTable.h"

/*************************************************************************/
// For arcball mouse motion - curtesy Shachar Fleishman                  
#include <gtbmouse.h>
using namespace gtbmouse;

static Mouse _mouse;
static View _view;
static Point3 _origin;
static double _scale_factor;
/*************************************************************************/


/*************************************************************************/
// The mesh to be rendered
/*************************************************************************/
TexturedMesh myMesh;

bool is_reading_mesh = false;


/*************************************************************************/
// For scaling, rotation and translation
/*************************************************************************/
float	xy_aspect;
int		last_x, last_y;

int		main_window;
GLfloat light_rotation0[16] = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1 };
GLfloat light_rotation1[16] = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1 };
GLfloat light_rotation2[16] = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1 };
GLfloat light_rotation3[16] = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1 };




/*************************************************************************/
// Live variables passed into GLUI
/*************************************************************************/
//int   wireframe = 0;
int		live_light0_enabled = 0;
int		live_light1_enabled = 0;
float	live_light0_intensity = 0.0;
float	live_light1_intensity = 0;
int		live_light2_enabled = 0;
float	live_light2_intensity = 0.0;
int		live_light3_enabled = 0;
float	live_light3_intensity = 0.0;
int		live_MPII_light_enabled = 1;


int		live_polygon_mode = 0;
int		live_balls_mode = 4;
int		live_cull_back = 0;
float	live_point_size = 1.0f;
float	live_circle_radius = 0.5f;
int		live_circle_width_segments = 10;
int 	live_circle_height_segments = 10;

int		live_nav_wnd_enabled = 1;
int		live_nav_wnd_show = 1;
int		live_smooth_shading = 1;
int		live_num_vertices = 0;
int		live_num_faces = 0;

int		live_camera_mode = 0;

int		live_color_index = 131;

int		live_clear_color = 1;

int		live_specular = 1;
int		live_render_bounding_box = 0;
int		live_shininess = 60;
float	live_Ks = 0.4;
float	live_min_edge_length = 0.0f;

int		selected_vertex = -1;
Box		mesh_bb;

int		live_use_mesh_colors = 0;


/*************************************************************************/
// Pointers to controls
/*************************************************************************/
GLUI			*glui, *glui2, *glui_io;
GLUI_Spinner    *light0_spinner, *light1_spinner, *light2_spinner, *light3_spinner, *circle_radius_spinner;
GLUI_Spinner	*circle_width_segment_spinner, *circle_height_segment_spinner;
GLUI_Panel      *polygon_panel;
GLUI_Checkbox   *enable_nav_wnd_checkbox; 
GLUI_Checkbox   *show_nav_wnd_checkbox;
GLUI_Listbox	*file_listbox;
GLUI_EditText   *numVertBox, *numFacesBox, *minEdgeLength;
GLUI_EditText   *inFileEdit, *vrml_box, *ply2_box, *smpl_box, *ppm_box, *anchsFilename, *smf_box;
GLUI_Rollout	*geo_p;
GLUI_EditText	*view_filename_box;
GLUI_RadioGroup *camera_radio;


/*************************************************************************/
// User IDs fpr callbacks
/*************************************************************************/
#define LIGHT0_ENABLED_ID    200
#define LIGHT1_ENABLED_ID    201
#define LIGHT0_INTENSITY_ID  250
#define LIGHT1_INTENSITY_ID  251
#define LIGHT2_ENABLED_ID    202
#define LIGHT2_INTENSITY_ID  252
#define LIGHT3_ENABLED_ID    203
#define LIGHT3_INTENSITY_ID  253
#define LIGHT_MPII_ENABLED_ID 254


#define NAV_WND_ENABLE_ID    300
#define NAV_WND_SHOW_ID      302
#define RESET_VIEW_ID		 400

#define FILE_EDIT_ID		 501

#define RELOAD_ID			 600


#define EXPORT_VRML_ID		 700
#define EXPORT_PLY2_ID		 701
#define CAPTURE_SCREEN_ID    702
#define EXPORT_SMPL_ID		 703
#define EXPORT_SMF_ID		 704
#define REVERSE_NORMALS      705


#define CAMERA_CHANGED_ID	 821


#define SAVE_VIEW_ID	     1000
#define LOAD_VIEW_ID		 1001



/*************************************************************************/
// Misc. global variables
/*************************************************************************/
GLfloat light0_ambient[] =  {0.1f, 0.1f, 0.3f, 1.0f};
GLfloat light0_diffuse[] =  {.6f, .6f, 1.0f, 1.0f};
GLfloat light0_position[] = {.5f, .5f, 1.0f, 0.0f};
GLfloat light0_specular[] =  {1.0f, 1.0f, 1.0f, 1.0f};

GLfloat light1_ambient[] =  {0.1f, 0.1f, 0.1f, 1.0f};
//GLfloat light1_diffuse[] =  {.9f, .6f, 0.0f, 1.0f};
GLfloat light1_diffuse[] =  {.917f, .621f, 0.0703f, 1.0f};
GLfloat light1_position[] = {.5f, .5f, 1.0f, 0.0f};
GLfloat light1_specular[] =  {1.0f, 1.0f, 1.0f, 1.0f};


GLfloat light2_ambient[] =  {0.1f, 0.1f, 0.1f, 1.0f};
GLfloat light2_diffuse[] =  {1.0f, 1.0f, 1.0f, 1.0f};
GLfloat light2_position[] = {.5f, .5f, 1.0f, 0.0f};
GLfloat light2_specular[] =  {1.0f, 1.0f, 1.0f, 1.0f};

GLfloat light3_ambient[] =  {0.05f, 0.05f, 0.05f, 1.0f};
GLfloat light3_diffuse[] =  {173/256.0f, 0/256.0f, 35/256.0f, 1.0f};
GLfloat light3_position[] = {.5f, .5f, 1.0f, 0.0f};
GLfloat light3_specular[] =  {1.0f, 1.0f, 1.0f, 1.0f};

GLfloat	specref[]		= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat	no_specref[]	= { 0.0f, 0.0f, 0.0f, 1.0f };

GLfloat clear_color_white[] = {1.0f, 1.0f, 1.0f, 1.0f};
GLfloat clear_color_gray[] =  {.95f, .95f, .95f, 1.0f};


// MPII lighting
GLfloat pos1[] = { 0.1, 0.1, -0.02, 0.0};
GLfloat pos2[] = {-0.1, 0.1, -0.02, 0.0};
GLfloat pos3[] = { 0.0, 0.0, 0.1, 0.0};
GLfloat col1[] = {.05, .05, .4, 1.0};
GLfloat col2[] = {.05, .05, .4, 1.0};
//GLfloat col2[] = {.6, .05, .05, 1.0};
GLfloat col3[] = {1.0, 1.0, 1.0, 1.0};

 

/*************************************************************************/
// Functions headers
/*************************************************************************/
void myGlutDisplay( void );
void SetupRC(void);
void control_cb( int control );
void myGlutKeyboard(unsigned char Key, int x, int y);
void myGlutSpecialFunc(int Key, int x, int y);
void myGlutMenu( int value );
void myGlutIdle( void );
void myGlutMouse(int button, int button_state, int x, int y );
void myGlutMotion(int x, int y );
void myGlutReshape( int x, int y );
void myGlutVisible(int state);
void myGlutPassiveMotion(int x, int y);

bool LoadMesh(TexturedMesh& myMesh1, const char *filename);
void openMesh();

void SelectVertexUnderCursor(int x, int y);
bool RenderMeshToPPM(const char *filename);
char * RemoveExtension(char *filename);


///////////////////////////////////////////////////////////////////////////
//  Implementation
///////////////////////////////////////////////////////////////////////////

void SetupRC(void)
{
	glClearColor( clear_color_gray[0], clear_color_gray[1], clear_color_gray[2], clear_color_gray[3]);
	
	/****************************************/
	/*       Set up OpenGL lights           */
	/****************************************/
	
	glFrontFace(GL_CCW);		// Counter clock-wise polygons face out
	
	if (live_cull_back != 0)
		glEnable(GL_CULL_FACE);		
	else
		glDisable(GL_CULL_FACE);		
	
	
	// Enable color tracking
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_NORMALIZE);
	// Set Material properties to follow glColor values
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	
	glMaterialfv(GL_FRONT, GL_SPECULAR, no_specref);
	glMateriali(GL_FRONT,GL_SHININESS, 0);
	
	
	glEnable(GL_LIGHTING);
	
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
	glLightfv(GL_LIGHT0, GL_POSITION, light0_position);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light0_specular);

	if (!live_light0_enabled)
	{
		glDisable(GL_LIGHT0);
	}
	
	glEnable(GL_LIGHT1);
	glLightfv(GL_LIGHT1, GL_AMBIENT, light1_ambient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_diffuse);
	glLightfv(GL_LIGHT1, GL_POSITION, light1_position);
	glLightfv(GL_LIGHT1, GL_SPECULAR, light1_specular);
	if (!live_light1_enabled)
	{
		glDisable(GL_LIGHT1);
	}
	
	glEnable(GL_LIGHT2);
	glLightfv(GL_LIGHT2, GL_AMBIENT, light2_ambient);
	glLightfv(GL_LIGHT2, GL_DIFFUSE, light2_diffuse);
	glLightfv(GL_LIGHT2, GL_POSITION, light2_position);
	glLightfv(GL_LIGHT2, GL_SPECULAR, light2_specular);
	if (!live_light2_enabled)
	{
		glDisable(GL_LIGHT2);
	}
	
	glPointSize(live_point_size);


	glEnable(GL_LIGHT3);
	glLightfv(GL_LIGHT3, GL_AMBIENT, light3_ambient);
	glLightfv(GL_LIGHT3, GL_DIFFUSE, light3_diffuse);
	glLightfv(GL_LIGHT3, GL_POSITION, light3_position);
	glLightfv(GL_LIGHT3, GL_SPECULAR, light3_specular);
	if (!live_light3_enabled)
	{
		glDisable(GL_LIGHT3);
	}


	glEnable(GL_LIGHT4);    
	glLightfv(GL_LIGHT4,GL_POSITION, pos1);
	glLightfv(GL_LIGHT4,GL_DIFFUSE,  col1);
	glLightfv(GL_LIGHT4,GL_SPECULAR, col1);

	glEnable(GL_LIGHT5);  
	glLightfv(GL_LIGHT5,GL_POSITION, pos2);
	glLightfv(GL_LIGHT5,GL_DIFFUSE,  col2);
	glLightfv(GL_LIGHT5,GL_SPECULAR, col2);

	glEnable(GL_LIGHT6);  
	glLightfv(GL_LIGHT6,GL_POSITION, pos3);
	glLightfv(GL_LIGHT6,GL_DIFFUSE,  col3);
	glLightfv(GL_LIGHT6,GL_SPECULAR, col3);
	if (!live_MPII_light_enabled) {
		glDisable(GL_LIGHT4);
		glDisable(GL_LIGHT5);
		glDisable(GL_LIGHT6);
	}

	
	glPointSize(live_point_size);
	
	/****************************************/
	/*          Enable z-buferring          */
	/****************************************/
	
	glDepthFunc(GL_LESS);	
	glEnable(GL_DEPTH_TEST);
	


	glEnable(GL_POINT_SMOOTH);
	glEnable(GL_LINE_SMOOTH);
//	glEnable(GL_POLYGON_SMOOTH);
	glShadeModel(GL_SMOOTH);

//	glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);

//	glEnable(GL_BLEND);
//	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}



/***************************************** myGlutDisplay() *****************/
void myGlutDisplay( void )
{
	switch (live_clear_color)
	{
	case 0:
		glClearColor(clear_color_gray[0], clear_color_gray[1], clear_color_gray[2], clear_color_gray[3]);
		break;
	case 1:
		glClearColor(clear_color_white[0], clear_color_white[1], clear_color_white[2], clear_color_white[3]);
		break;
	default:
		break;
	}

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	if (live_specular)
	{
		GLfloat sp[4];
		sp[0] = live_Ks;
		sp[1] = live_Ks;
		sp[2] = live_Ks;
		sp[3] = 1.0f;

		glMaterialfv(GL_FRONT, GL_SPECULAR, sp);
		glMateriali(GL_FRONT,GL_SHININESS, 60);
	}
	else
	{
		glMaterialfv(GL_FRONT, GL_SPECULAR, no_specref);
		glMateriali(GL_FRONT,GL_SHININESS, 0);
	}

	
	if (live_cull_back != 0)
		glEnable(GL_CULL_FACE);		
	else
		glDisable(GL_CULL_FACE);
	
	
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glFrustum( -xy_aspect*.04, xy_aspect*.04, -.04, .04, .1, 15.0 );
	
	glMatrixMode( GL_MODELVIEW );
	
	glLoadIdentity();
	glMultMatrixf( light_rotation0 );
	glLightfv(GL_LIGHT0, GL_POSITION, light0_position);

	glLoadIdentity();
	glMultMatrixf( light_rotation1 );
	glLightfv(GL_LIGHT1, GL_POSITION, light1_position);

	glLoadIdentity();
	glMultMatrixf( light_rotation2 );
	glLightfv(GL_LIGHT2, GL_POSITION, light2_position);

	glLoadIdentity();
	glMultMatrixf( light_rotation3 );
	glLightfv(GL_LIGHT3, GL_POSITION, light3_position);



	// arcball
	_view.load_camera();

	
	switch(live_balls_mode)
	{
	case 0: // balls around all vertices
		glColor3f(1.0f, 0.0f, 0.0f);
		glPolygonMode(GL_FRONT,GL_FILL);
		myMesh.RenderVertexBalls(live_circle_radius, 
								 0, 
								 live_circle_width_segments,
								 live_circle_height_segments);
		glColor3f(1.0, 1.0, 1.0);
		break;
		
	case 1: // balls around boundary vertices
		glColor3f(1.0f, 1.0f, 0.5f);
		glPolygonMode(GL_FRONT,GL_FILL);
		myMesh.RenderVertexBalls(live_circle_radius, 
								 1, 
								 live_circle_width_segments,
								 live_circle_height_segments);
		glColor3f(1.0, 1.0, 1.0);
		break;

	case 2: 
		glColor3f(1.0f, 0.0f, 1.0f);
		glPolygonMode(GL_FRONT,GL_FILL);
		myMesh.RenderVertexBalls(live_circle_radius, 
								 2, 
								 live_circle_width_segments,
								 live_circle_height_segments);
		glColor3f(1.0, 1.0, 1.0);
		break;

	case 3:
		glColor3f(1.0f, 1.0f, 0.5f);
		glPolygonMode(GL_FRONT,GL_FILL);
		myMesh.RenderVertexBalls(live_circle_radius, 
							     3, 
								 live_circle_width_segments,
								 live_circle_height_segments);
		glColor3f(1.0, 1.0, 1.0);
		break;

	default:
		break;
	}
	
	
	glColor3f(1.0, 1.0, 1.0);
	
	// The actual mesh rendering
	switch (live_polygon_mode)
	{
	case 0: // solid
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		myMesh.RenderTriangles(live_smooth_shading != 0, 
								(live_use_mesh_colors)? 0 : 2, 
							   GetColorFromTable(live_color_index));
		break;
		
	case 1: // wireframe
		glLineWidth(1.0);
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
		myMesh.RenderTriangles(live_smooth_shading != 0, 
							   (live_use_mesh_colors)? 0 : 2,
							   GetColorFromTable(live_color_index));
		break;
		
	case 2: // wireframe + solid
		
		glColor3f(0.3,0.3,0.3);
		
		glDisable(GL_LIGHTING);
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);		
		myMesh.RenderTriangles(live_smooth_shading != 0, 2);
		
		glColor3f(1.0, 1.0, 1.0);
		glEnable(GL_LIGHTING);
		glEnable(GL_POLYGON_OFFSET_FILL);		
		glPolygonOffset(3.0, 2);
		
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		myMesh.RenderTriangles(live_smooth_shading != 0, 
							   (live_use_mesh_colors)? 0 : 2, 
							   GetColorFromTable(live_color_index));
		
		glDisable(GL_POLYGON_OFFSET_FILL);		
		
		break;
		
	case 3: // points
		if (live_balls_mode != 0)  // otherwise renders balls anyway...
		{
			glPolygonMode(GL_FRONT_AND_BACK,GL_POINT);
			glPointSize(live_point_size);
			myMesh.RenderPoints();
		}
		break;
		
	default:
		break;
	}



	// drawing the picked vertex:
	if (selected_vertex > -1 && selected_vertex < myMesh.GetNumberOfVertices())
	{
		Point3D picked_v = myMesh.GetVertex(selected_vertex);

		glColor3f(1.0f, 1.0f, 0.0f);

		glTranslatef(picked_v.GetX(), picked_v.GetY(), picked_v.GetZ());
		glutSolidSphere(live_circle_radius*1.2,      /* radius */ 
						live_circle_width_segments,	 /* slices */ 
						live_circle_width_segments	 /* stacks */ );
		glTranslatef(-picked_v.GetX(), -picked_v.GetY(), -picked_v.GetZ());

		glColor3f(1.0, 1.0, 1.0);
	}	
	
// Drawing the bounding box
	if (live_render_bounding_box)
	{
		glDisable(GL_LIGHTING);
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
		glColor3f(0.0, 1.0, 0.0);
		glLineWidth(2.0);

		GLfloat xmin = mesh_bb.GetLeft();
		GLfloat xmax = mesh_bb.GetRight();
		GLfloat ymin = mesh_bb.GetBack();
		GLfloat ymax = mesh_bb.GetFront();
		GLfloat zmin = mesh_bb.GetBottom();
		GLfloat zmax = mesh_bb.GetTop();

		
		glBegin(GL_QUADS);
			glVertex3f(xmin, ymin, zmin);
			glVertex3f(xmax, ymin, zmin);
			glVertex3f(xmax, ymin, zmax);
			glVertex3f(xmin, ymin, zmax);
			
			glVertex3f(xmax, ymin, zmin);
			glVertex3f(xmax, ymax, zmin);
			glVertex3f(xmax, ymax, zmax);
			glVertex3f(xmax, ymin, zmax);
			
			glVertex3f(xmax, ymax, zmax);
			glVertex3f(xmax, ymax, zmin);
			glVertex3f(xmin, ymax, zmin);
			glVertex3f(xmin, ymax, zmax);
			
			glVertex3f(xmin, ymin, zmin);
			glVertex3f(xmin, ymin, zmax);
			glVertex3f(xmin, ymax, zmax);
			glVertex3f(xmin, ymax, zmin);

			glVertex3f(xmin, ymin, zmax);
			glVertex3f(xmax, ymin, zmax);
			glVertex3f(xmax, ymax, zmax);
			glVertex3f(xmin, ymax, zmax);

			glVertex3f(xmin, ymin, zmin);
			glVertex3f(xmin, ymax, zmin);
			glVertex3f(xmax, ymax, zmin);
			glVertex3f(xmax, ymin, zmin);
		glEnd();

		glLineWidth(1.0);
	}


	glEnable( GL_LIGHTING );
	
	
	glutSwapBuffers(); 
}


/**************************************** main() ********************/
void main(int argc, char* argv[])
{
	/****************************************/
	/*   Initialize GLUT and create window  */
	/****************************************/
	
	
	glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
	glutInitWindowPosition( 50, 50 );
	glutInitWindowSize( 800, 600 );
	
	
	main_window = glutCreateWindow( "Olga's Mesh Renderer Version 1.1" );
	glutDisplayFunc( myGlutDisplay );
	GLUI_Master.set_glutReshapeFunc( myGlutReshape );  
	GLUI_Master.set_glutKeyboardFunc( myGlutKeyboard );
	GLUI_Master.set_glutSpecialFunc( myGlutSpecialFunc );
	GLUI_Master.set_glutMouseFunc( myGlutMouse );
	
	glutPassiveMotionFunc( myGlutPassiveMotion );
	
	//	glutVisibilityFunc ( myGlutVisible );
	glutMotionFunc( myGlutMotion ); 
	
	// Setup all the OpenGL preliminaries....
	SetupRC();
	
	
	/****************************************/
	/*         Here's the GLUI code         */
	/****************************************/
	
	cout << "*****************The amazing GLUI mesh renderer!!*****************" << endl << endl;
	
	
	/*** Create the top subwindow ***/
	glui_io = GLUI_Master.create_glui_subwindow( main_window, 
		GLUI_SUBWINDOW_TOP );
	glui_io->set_main_gfx_window( main_window );
	

	
	//	glui_io->add_column(false);
	inFileEdit = glui_io->add_edittext("Enter filname: ", GLUI_EDITTEXT_TEXT, NULL, FILE_EDIT_ID, control_cb);
	inFileEdit->set_w(200);
	inFileEdit->disable();

	glui_io->add_column(false);
	glui_io->add_button("Load File", FILE_EDIT_ID, control_cb);
	
	
	glui_io->add_column(false);
	GLUI_Button *ReloadButton = glui_io->add_button("Reload", RELOAD_ID, control_cb);

	
	/*** Create the side subwindow ***/
	glui = GLUI_Master.create_glui_subwindow( main_window, GLUI_SUBWINDOW_RIGHT );
	
	
	/*** Info panel ***/
	GLUI_Rollout *info_rollout = glui->add_rollout("Info", false);
	char verticesText[50] = "";
	char facesText[50] = "";
	
	glui->add_statictext_to_panel(info_rollout, "Num. vertices: ");
	glui->add_statictext_to_panel(info_rollout, "Num. faces: ");
	glui->add_statictext_to_panel(info_rollout, "");
	glui->add_statictext_to_panel(info_rollout, "Mininum edge length: ");

	
	glui->add_column_to_panel(info_rollout, false);
	
	numVertBox = glui->add_edittext_to_panel(info_rollout, 
											 "",
											 GLUI_EDITTEXT_INT, 
											 &live_num_vertices);
	numVertBox->disable();	
	
	numFacesBox = glui->add_edittext_to_panel(info_rollout, 
											  "",
											  GLUI_EDITTEXT_INT, 
											  &live_num_faces);
	numFacesBox->disable();	


	minEdgeLength = glui->add_edittext_to_panel(info_rollout, 
											      "",
											      GLUI_EDITTEXT_FLOAT, 
											      &live_min_edge_length);
	minEdgeLength->disable();	
	

	
	//// Here, we load the initial mesh/////////////////////////////////////////
	LoadMesh(myMesh,"");
	numVertBox->set_int_val(myMesh.GetNumberOfVertices());
	numFacesBox->set_int_val(myMesh.GetNumberOfTriangles());
	////////////////////////////////////////////////////////////////////////////


	/**** Export and capture handling panel *****************************************************/
	GLUI_Rollout *formats_rollout = glui->add_rollout("Export files", false);

	vrml_box = glui->add_edittext_to_panel(formats_rollout, "Filename (*.wrl):", GLUI_EDITTEXT_TEXT);
	vrml_box->set_alignment(GLUI_ALIGN_RIGHT);
	ply2_box = glui->add_edittext_to_panel(formats_rollout, "Filename (*.ply2):", GLUI_EDITTEXT_TEXT);
	ply2_box->set_alignment(GLUI_ALIGN_RIGHT);
	smpl_box = glui->add_edittext_to_panel(formats_rollout, "Filename (*.smpl):", GLUI_EDITTEXT_TEXT);
	smpl_box->set_alignment(GLUI_ALIGN_RIGHT);
	smf_box = glui->add_edittext_to_panel(formats_rollout, "Filename (*.smf):", GLUI_EDITTEXT_TEXT);
	smf_box->set_alignment(GLUI_ALIGN_RIGHT);
	
	glui->add_statictext_to_panel(formats_rollout, "");
	ppm_box = glui->add_edittext_to_panel(formats_rollout, "Filename (*.ppm):", GLUI_EDITTEXT_TEXT);
	ppm_box->set_alignment(GLUI_ALIGN_RIGHT);


	glui->add_column_to_panel(formats_rollout, false);
	
	GLUI_Button *button_vrml = glui->add_button_to_panel(formats_rollout, "Export to VRML 2.0", EXPORT_VRML_ID, control_cb);
	button_vrml->set_alignment(GLUI_ALIGN_LEFT);
	GLUI_Button *button_ply2 = glui->add_button_to_panel(formats_rollout, "Export to PLY2", EXPORT_PLY2_ID, control_cb);
	button_ply2->set_alignment(GLUI_ALIGN_LEFT);
	GLUI_Button *button_smpl = glui->add_button_to_panel(formats_rollout, "Export to SMPL", EXPORT_SMPL_ID, control_cb);
	button_smpl->set_alignment(GLUI_ALIGN_LEFT);
	GLUI_Button *button_smf = glui->add_button_to_panel(formats_rollout, "Export to SMF", EXPORT_SMF_ID, control_cb);
	button_smf->set_alignment(GLUI_ALIGN_LEFT);

	glui->add_statictext_to_panel(formats_rollout, "");
	GLUI_Button *button_ppm = glui->add_button_to_panel(formats_rollout, "Capture screen", CAPTURE_SCREEN_ID, control_cb);
	button_ppm->set_alignment(GLUI_ALIGN_LEFT);

	glui->add_separator_to_panel(formats_rollout);
	glui->add_button_to_panel(formats_rollout, "Reverse normals", REVERSE_NORMALS, control_cb);


	
/**** Rendering modes panel ****************************************************/
	polygon_panel = glui->add_rollout( "Rendering modes", true );

	glui->add_checkbox_to_panel(polygon_panel, 
								"Use specular material", 
								&live_specular);

	GLUI_Spinner *Ks_spinner = glui->add_spinner_to_panel(polygon_panel, 
														  "Ks:",
														  GLUI_SPINNER_FLOAT, 
														  &live_Ks);
	Ks_spinner->set_float_limits(0.0, 1.0);

	GLUI_Spinner *shininess_spinner = glui->add_spinner_to_panel(polygon_panel, 
															  "Shininess:",
															  GLUI_SPINNER_INT, 
															  &live_shininess);
	shininess_spinner->set_int_limits(0, 128);

	GLUI_EditText *currentColor = glui->add_edittext_to_panel(polygon_panel,
															  "Current color:",
															  GLUI_EDITTEXT_INT,
															  &live_color_index);


	glui->add_checkbox_to_panel(polygon_panel, 
								"Show bounding box", 
								&live_render_bounding_box);

	
	GLUI_Rollout *poly_p = glui->add_rollout_to_panel(polygon_panel, "Polygon modes");
	
	// Radio buttons for different polygon modes:
	GLUI_RadioGroup *polygon_radio =  glui->add_radiogroup_to_panel(poly_p, 
		&live_polygon_mode);
	
	glui->add_radiobutton_to_group(polygon_radio, "Solid");
	glui->add_radiobutton_to_group(polygon_radio, "Wireframe");
	glui->add_radiobutton_to_group(polygon_radio, "Solid + wireframe");
	glui->add_radiobutton_to_group(polygon_radio, "Points");
	
	// Point size spinner
	GLUI_Spinner *point_size_spinner = glui->add_spinner_to_panel(poly_p, 
																  "Point size:",
																  GLUI_SPINNER_FLOAT, 
																  &live_point_size);
	
	GLfloat point_size_range[2];
	glGetFloatv(GL_POINT_SIZE_RANGE, point_size_range);
	point_size_spinner->set_float_limits(point_size_range[0], point_size_range[1]);
	point_size_spinner->set_alignment( GLUI_ALIGN_LEFT );
	
	
	// camera models
	GLUI_Rollout *camera_p = glui->add_rollout_to_panel(polygon_panel, "Camera model");
	
	// Radio buttons for different camera models
	camera_radio =  glui->add_radiogroup_to_panel(camera_p, 
		&live_camera_mode, CAMERA_CHANGED_ID, control_cb);
	
	GLUI_RadioButton * camera_btn_persp = glui->add_radiobutton_to_group(camera_radio, "Perspective");
	GLUI_RadioButton * camera_btn_ortho = glui->add_radiobutton_to_group(camera_radio, "Orthographic");

	if (live_camera_mode == 0) {
		_view.camera().set_camera_type(Camera::PERSPECTIVE);
	}
	else {
		_view.camera().set_camera_type(Camera::ORTHOGRAPHIC);
		_view.camera().set_ortho_scaling(0);
	}

	
	
	GLUI_Rollout *balls_p = glui->add_rollout_to_panel(polygon_panel, "Balls for vertices");
	
	// Radio buttons for different balls-around-vertices modes:
	GLUI_RadioGroup *balls_radio =  glui->add_radiogroup_to_panel(balls_p, 
		&live_balls_mode);
	
	glui->add_radiobutton_to_group(balls_radio, "Show vertices as balls");
	glui->add_radiobutton_to_group(balls_radio, "Show boundary vertices as balls");
	glui->add_radiobutton_to_group(balls_radio, "Show isolated vertices as balls");
	glui->add_radiobutton_to_group(balls_radio, "Show hanging vertices as balls");
	glui->add_radiobutton_to_group(balls_radio, "No balls");
	
	// Circle radius spinner
	// calculate initial radius:
	circle_radius_spinner = glui->add_spinner_to_panel(balls_p, 
													   "Circle radius:",
													   GLUI_SPINNER_FLOAT, 
													   &live_circle_radius);
	circle_radius_spinner->set_float_limits(0.0f, 100*live_circle_radius);
	circle_radius_spinner->set_alignment( GLUI_ALIGN_LEFT );


	circle_width_segment_spinner = glui->add_spinner_to_panel(balls_p, 
													   "Width segments:",
													   GLUI_SPINNER_INT, 
													   &live_circle_width_segments);
	circle_width_segment_spinner->set_float_limits(3, 30);
	circle_width_segment_spinner->set_alignment( GLUI_ALIGN_LEFT );

	circle_height_segment_spinner = glui->add_spinner_to_panel(balls_p, 
													   "Height segments:",
													   GLUI_SPINNER_INT, 
													   &live_circle_height_segments);
	circle_height_segment_spinner->set_float_limits(3, 30);
	circle_height_segment_spinner->set_alignment( GLUI_ALIGN_LEFT );

	
	
	// Backface culling check box
	glui->add_checkbox_to_panel(polygon_panel, 
								"Cull back faces", 
								&live_cull_back);
	
	
	// Smooth.flat shading
	glui->add_checkbox_to_panel(polygon_panel, 
								"Gouraud shading", 
								&live_smooth_shading);


	// Handling views
	GLUI_Rollout *views_p = glui->add_rollout_to_panel(polygon_panel, "Load/Save views", false);
	
	view_filename_box = glui->add_edittext_to_panel(views_p,
												   "Filename",
												   GLUI_EDITTEXT_TEXT,
												   NULL,
												   NULL,
												   NULL);
	glui->add_button_to_panel(views_p, "Save view", SAVE_VIEW_ID, control_cb);
	glui->add_button_to_panel(views_p, "Load view", LOAD_VIEW_ID, control_cb);

	
	
	
	/******** Add some controls for lights ********************************************/
	
	GLUI_Rollout *roll_lights = glui->add_rollout( "Lights", false );
	
	GLUI_Panel *light0 = glui->add_panel_to_panel( roll_lights, "Light 1 (blue)" );
	GLUI_Panel *light1 = glui->add_panel_to_panel( roll_lights, "Light 2 (golden)" );
	GLUI_Panel *light2 = glui->add_panel_to_panel( roll_lights, "Light 3 (white)" );
	GLUI_Panel *light3 = glui->add_panel_to_panel( roll_lights, "Light 4 (red)" );
	GLUI_Panel *light_mpii = glui->add_panel_to_panel( roll_lights, "MPII lighting" );
	
	glui->add_checkbox_to_panel( light0, "Enabled", &live_light0_enabled,
								 LIGHT0_ENABLED_ID, control_cb );
	light0_spinner = 
		glui->add_spinner_to_panel( light0, "Intensity:", GLUI_SPINNER_FLOAT,
									&live_light0_intensity, LIGHT0_INTENSITY_ID, control_cb );
	light0_spinner->set_float_limits( 0.0, 1.0 );
	if (! live_light0_enabled)
		light0_spinner->disable();
	
	
	glui->add_checkbox_to_panel( light1, "Enabled", &live_light1_enabled,
								 LIGHT1_ENABLED_ID, control_cb );
	light1_spinner = 
		glui->add_spinner_to_panel( light1, "Intensity:", GLUI_SPINNER_FLOAT,
									&live_light1_intensity, LIGHT1_INTENSITY_ID, control_cb );
	light1_spinner->set_float_limits( 0.0, 1.0 );
	if (! live_light1_enabled)
		light1_spinner->disable();

	
	
	glui->add_checkbox_to_panel( light2, "Enabled", &live_light2_enabled,
								 LIGHT2_ENABLED_ID, control_cb );
	light2_spinner = 
		glui->add_spinner_to_panel( light2, "Intensity:", GLUI_SPINNER_FLOAT,
									&live_light2_intensity, LIGHT2_INTENSITY_ID, control_cb );
	light2_spinner->set_float_limits( 0.0, 1.0 );
	if (! live_light2_enabled)
		light2_spinner->disable();


	glui->add_checkbox_to_panel( light3, "Enabled", &live_light3_enabled,
								 LIGHT3_ENABLED_ID, control_cb );
	light3_spinner = 
		glui->add_spinner_to_panel( light3, "Intensity:", GLUI_SPINNER_FLOAT,
									&live_light3_intensity, LIGHT3_INTENSITY_ID, control_cb );
	light3_spinner->set_float_limits( 0.0, 1.0 );
	if (! live_light3_enabled)
		light3_spinner->disable();

	glui->add_checkbox_to_panel( light_mpii, "Enabled", &live_MPII_light_enabled,
								 LIGHT_MPII_ENABLED_ID, control_cb );

	GLUI_Panel *bgnd_p = glui->add_panel_to_panel(roll_lights, "Background color");
	GLUI_RadioGroup *bgnd_radio =  glui->add_radiogroup_to_panel(bgnd_p, &live_clear_color);
	glui->add_radiobutton_to_group(bgnd_radio, "Light gray");
	glui->add_radiobutton_to_group(bgnd_radio, "White");


	
	
	
	/********* Navigating Window Disable/Enable ****************************************/

	GLUI_Rollout *roll_windows = glui->add_rollout( "Windows", false );
	
	show_nav_wnd_checkbox =
		glui->add_checkbox_to_panel(roll_windows,
									"Show lights navigation window", 
									&live_nav_wnd_show,
									NAV_WND_SHOW_ID, 
									control_cb);
	
	enable_nav_wnd_checkbox =
		glui->add_checkbox_to_panel(roll_windows,
									"Enable lights navigation window", 
									&live_nav_wnd_enabled,
									NAV_WND_ENABLE_ID, 
									control_cb);
	
	glui->add_statictext( "" );
	glui->add_statictext( "" );
	
	
	glui->add_button("Reset\nview", RESET_VIEW_ID, control_cb);
	
	glui->add_statictext( "" );
	glui->add_statictext( "" );
	
	/****** A 'quit' button *****/
	glui->add_button( "Quit", 0,(GLUI_Update_CB)exit );
	
	
	
	/**** Link windows to GLUI, and register idle callback ******/
	
	glui->set_main_gfx_window( main_window );
	
	
	
	/*** Create the bottom (navigation) subwindow *****************************************/


	glui2 = GLUI_Master.create_glui_subwindow( main_window, 
		GLUI_SUBWINDOW_BOTTOM );
	glui2->set_main_gfx_window( main_window );

	GLUI_Rotation *light_rot0 = glui2->add_rotation( "Blue Light", light_rotation0);
	light_rot0->set_spin( .82 );
	glui2->add_column( false );

	GLUI_Rotation *light_rot1 = glui2->add_rotation( "Golden Light", light_rotation1);
	light_rot1->set_spin( .82 );
	glui2->add_column( false );

	GLUI_Rotation *light_rot2 = glui2->add_rotation( "White Light", light_rotation2);
	light_rot2->set_spin( .82 );
	glui2->add_column( false );


	GLUI_Rotation *light_rot3 = glui2->add_rotation( "Red Light", light_rotation3);
	light_rot3->set_spin( .82 );
	glui2->add_column( false );

	
	/**** We register the idle callback with GLUI, *not* with GLUT ****/
	//	GLUI_Master.set_glutIdleFunc( myGlutIdle );
	
	/**** Regular GLUT main loop ****/
	
	glutMainLoop();
}



/**************************************** control_cb() *******************/
/* GLUI control callback                                                 */
/*************************************************************************/
void control_cb( int control )
{
	switch (control)
	{
	case LIGHT0_ENABLED_ID :
		if ( live_light0_enabled ) 
		{
			glEnable( GL_LIGHT0 );
			light0_spinner->enable();
		}
		else 
		{
			glDisable( GL_LIGHT0 ); 
			light0_spinner->disable();
		}
		break;
		
	case LIGHT1_ENABLED_ID :
		if ( live_light1_enabled ) 
		{
			glEnable( GL_LIGHT1 );
			light1_spinner->enable();
		}
		else 
		{
			glDisable( GL_LIGHT1 ); 
			light1_spinner->disable();
		}
		break;
		
	case LIGHT0_INTENSITY_ID :
		{
			float v[] = { light0_diffuse[0],  light0_diffuse[1],
				light0_diffuse[2],  light0_diffuse[3] };
			
			v[0] *= live_light0_intensity;
			v[1] *= live_light0_intensity;
			v[2] *= live_light0_intensity;
			
			glLightfv(GL_LIGHT0, GL_DIFFUSE, v );
		}
		
		break;
		
	case LIGHT1_INTENSITY_ID :
		{
			float v[] = { light1_diffuse[0],  light1_diffuse[1],
				light1_diffuse[2],  light1_diffuse[3] };
			
			v[0] *= live_light1_intensity;
			v[1] *= live_light1_intensity;
			v[2] *= live_light1_intensity;
			
			glLightfv(GL_LIGHT1, GL_DIFFUSE, v );
		}
		
		break;
		
	case LIGHT2_ENABLED_ID :
		if ( live_light2_enabled ) 
		{
			glEnable( GL_LIGHT2 );
			light2_spinner->enable();
		}
		else 
		{
			glDisable( GL_LIGHT2 ); 
			light2_spinner->disable();
		}
		break;
		
	case LIGHT2_INTENSITY_ID :
		{
			float v[] = { light2_diffuse[0],  light2_diffuse[1],
				light2_diffuse[2],  light2_diffuse[3] };
			
			v[0] *= live_light2_intensity;
			v[1] *= live_light2_intensity;
			v[2] *= live_light2_intensity;
			
			glLightfv(GL_LIGHT2, GL_DIFFUSE, v );
		}
		
		break;
		
	case LIGHT3_ENABLED_ID :
		if ( live_light3_enabled ) 
		{
			glEnable( GL_LIGHT3 );
			light3_spinner->enable();
		}
		else 
		{
			glDisable( GL_LIGHT3 ); 
			light3_spinner->disable();
		}
		break;
		
	case LIGHT3_INTENSITY_ID :
		{
			float v[] = { light3_diffuse[0],  light3_diffuse[1],
				light3_diffuse[2],  light3_diffuse[3] };
			
			v[0] *= live_light3_intensity;
			v[1] *= live_light3_intensity;
			v[2] *= live_light3_intensity;
			
			glLightfv(GL_LIGHT3, GL_DIFFUSE, v );
		}
		
		break;

	case LIGHT_MPII_ENABLED_ID:
		if ( live_MPII_light_enabled ) 
		{
			glEnable( GL_LIGHT4 );
			glEnable( GL_LIGHT5 );
			glEnable( GL_LIGHT6 );
		}
		else 
		{
			glDisable( GL_LIGHT4 ); 
			glDisable( GL_LIGHT5 ); 
			glDisable( GL_LIGHT6 ); 
		}
		break;

	case NAV_WND_ENABLE_ID:
		{
			if (live_nav_wnd_enabled)
				glui2->enable();
			else
				glui2->disable();
		}
		break;
		
	case NAV_WND_SHOW_ID:
		{
			if (live_nav_wnd_show)
				glui2->show();
			else
				glui2->hide();
		}
		break;
		
	case RESET_VIEW_ID:
		{
			memset(light_rotation0, 0, sizeof(light_rotation0));
			light_rotation0[0] = light_rotation0[5] = light_rotation0[10] = light_rotation0[15] = 1.0f;
			
			memcpy(light_rotation1, light_rotation0, sizeof(light_rotation0));
			memcpy(light_rotation2, light_rotation0, sizeof(light_rotation0));

			glui->sync_live();
			glui2->sync_live();
			
			Vector3 towards(0,0,-1);
			Vector3 up(0, 1, 0);
			Box bb = myMesh.GetBoundingBox();
			_view.camera().reset(_origin - Vector3(0,0,-1.5), towards, up, M_PI/6.0, M_PI/6.0/xy_aspect, 0.1, 14.0); 
			_scale_factor = bb.GetMaxEdgeLength() * 0.01;
		}	
		break;

	case REVERSE_NORMALS:
		myMesh.ReverseAllNormals();
		myMesh.SetMesh2Render();
		break;

			
	case FILE_EDIT_ID:
		
		openMesh();
		//LoadMesh(myMesh, inFileEdit->get_text());
		numVertBox->set_int_val(myMesh.GetNumberOfVertices());
		numFacesBox->set_int_val(myMesh.GetNumberOfTriangles());
		live_circle_radius = myMesh.GetAverageEdgeLength() / 10.0f;
		circle_radius_spinner->set_float_limits(0.0f, 100*live_circle_radius);
		
		glui->sync_live();
		glui2->sync_live();
		break;
		

	case RELOAD_ID: 
		LoadMesh(myMesh, inFileEdit->get_text());
		glui->sync_live();
		break;


	case EXPORT_VRML_ID:
		{
			ofstream of(vrml_box->get_text());

			if (of != NULL) {
				myMesh.WriteToVRML(of);
			}

			of.close();
		}
		break;

	case EXPORT_SMPL_ID:
		{
			ofstream of(smpl_box->get_text());

			if (of != NULL)
				myMesh.WriteToSimpleFormat(of);

			of.close();
		}
		break;

	case EXPORT_SMF_ID:
		{
			ofstream of(smf_box->get_text());

			if (of != NULL)
				myMesh.WriteToSMFFormat(of);

			of.close();
		}
		break;

	case EXPORT_PLY2_ID:
		{
			ofstream of(ply2_box->get_text());

			if (of != NULL)
				myMesh.WriteToPly2Format(of);

			of.close();
		}
		break;


	case CAPTURE_SCREEN_ID:
		RenderMeshToPPM(ppm_box->get_text());
		break;

	case SAVE_VIEW_ID:
		_view.camera().write(view_filename_box->get_text());
		break;

	case LOAD_VIEW_ID:
		_view.camera().read(view_filename_box->get_text());
		if (_view.camera().get_camera_type()==Camera::PERSPECTIVE) {
			camera_radio->set_int_val(0);
		}
		else {
			camera_radio->set_int_val(1);
		}
		glui->sync_live();

		break;	

	case CAMERA_CHANGED_ID: {
		if (live_camera_mode == 0) {
			_view.camera().set_camera_type(Camera::PERSPECTIVE);
			control_cb(RESET_VIEW_ID);
		} else if (live_camera_mode == 1) {
			_view.camera().set_camera_type(Camera::ORTHOGRAPHIC);
			_view.camera().set_ortho_scaling(0);
			control_cb(RESET_VIEW_ID);
		}
		}
		break;

		
	default:
		break;
	}
}



/**************************************** myGlutKeyboard() **********/
void myGlutKeyboard(unsigned char Key, int x, int y)
{
	switch(Key)
	{
	case '+':
		live_color_index++;
		live_color_index %= ncolors;
		cout << "Current color: " << live_color_index << endl;
		glui->sync_live();
		break;

	case '-':
		live_color_index--;
		live_color_index %= ncolors;
		cout << "Current color: " << live_color_index << endl;
		glui->sync_live();
		break;
	case 'f':
		myMesh.FixOrientation();
		break;
		
	case 27: 
	case 'q':
		exit(0);
		break;
	};
	
	glutPostRedisplay();
}

/**************************************** myGlutSpecialFunc() **********/
void myGlutSpecialFunc(int Key, int x, int y)
{
	glutPostRedisplay();
}


/***************************************** myGlutMenu() ***********/
void myGlutMenu( int value )
{
	myGlutKeyboard( value, 0, 0 );
}



/***************************************** myGlutIdle() ***********/
void myGlutIdle( void )
{
/* According to the GLUT specification, the current window is 
undefined during an idle callback.  So we need to explicitly change
	it if necessary */
	if ( glutGetWindow() != main_window ) 
		glutSetWindow(main_window);  
	
		/*  GLUI_Master.sync_live_all();  -- not needed - nothing to sync in this
	application  */
	
	glutPostRedisplay();
}


/***************************************** myGlutMouse() **********/
void myGlutMouse(int button, int button_state, int x, int y )
{
	// for arcball
	_mouse.set_button_state(button, button_state);
    _mouse.set_position(x, y);
	
	if ( button == GLUT_LEFT_BUTTON && button_state == GLUT_DOWN ) 
	{
		last_x = x;
		last_y = y;
	}
	else if ( button = GLUT_MIDDLE_BUTTON && button_state == GLUT_DOWN )
	{
		SelectVertexUnderCursor(x, y);
		glutPostRedisplay();
	}

}

void myGlutPassiveMotion(int x, int y)
{
	_mouse.set_position(x, y);
}



/***************************************** myGlutMotion() **********/
void myGlutMotion(int x, int y )
{	
	// for arcball
    _mouse.set_position(x, y);
    bool redraw = false;
    if (_mouse.is_button_pressed(GLUT_LEFT_BUTTON)) {
        int dx = _mouse.dx();
        int dy = _mouse.dy();
        if (keyboard::is_shift_pressed()) {
            _view.translate_world(_origin, dx, dy);
            redraw = true;
        } else if (keyboard::is_control_pressed()) {
            _view.scale_world(_scale_factor * dy);
            redraw = true;
        } else {
            _view.rotate_world(_origin, dx, dy);
            redraw = true;
        }
    }
    if (redraw) {
        glutPostRedisplay();
    }	
}


/**************************************** myGlutReshape() *************/
void myGlutReshape( int x, int y )
{
	int tx, ty, tw, th;
	GLUI_Master.get_viewport_area( &tx, &ty, &tw, &th );
	glViewport( tx, ty, tw, th );
	
	xy_aspect = (float)tw / (float)th;
	
	// for arcball
    _view.set_viewport(Viewport(tx, ty, tw, th));
	
	glutPostRedisplay();
}


/************************************************** draw_axes() **********/
/* Disables lighting, then draws RGB axes                                */
#ifdef AXES
void draw_axes( float scale )
{
	glDisable( GL_LIGHTING );
	
	glPushMatrix();
	glScalef( scale, scale, scale );
	
	glBegin( GL_LINES );
	
	glColor3f( 1.0, 0.0, 0.0 );
	glVertex3f( .8f, 0.05f, 0.0 );  glVertex3f( 1.0, 0.25f, 0.0 ); /* Letter X */
	glVertex3f( 0.8f, .25f, 0.0 );  glVertex3f( 1.0, 0.05f, 0.0 );
	glVertex3f( 0.0, 0.0, 0.0 );  glVertex3f( 1.0, 0.0, 0.0 ); /* X axis      */
	
	glColor3f( 0.0, 1.0, 0.0 );
	glVertex3f( 0.0, 0.0, 0.0 );  glVertex3f( 0.0, 1.0, 0.0 ); /* Y axis      */
	
	glColor3f( 0.0, 0.0, 1.0 );
	glVertex3f( 0.0, 0.0, 0.0 );  glVertex3f( 0.0, 0.0, 1.0 ); /* Z axis    */
	glEnd();
	
	glPopMatrix();
	
	glEnable( GL_LIGHTING );
}
#endif


bool LoadMesh(TexturedMesh & myMesh1, const char *filename)
{
	myMesh1.Clear();
	live_circle_radius = 0.5f;
	
	if (strlen(filename) <= 0)
	{
		return false;
	}
	
	ifstream inFile(filename, ios::in|ios::nocreate);
	
	is_reading_mesh = true;
	
	if (inFile.fail())
	{
		cout << "\nerror reading file " << filename << endl;
		is_reading_mesh = false;
		inFile.close();
		return false;
	}
	
	
	if (strcmp(strstr(filename, "."), ".smpl") == 0)
	{
		myMesh1.ReadFromSimpleFormat(inFile);
		cout << "\nsuccess reading SIMPLE file " << filename << endl;
		glui->sync_live();
	}
	else if (strcmp(strstr(filename, "."), ".ply2") == 0)
	{
		if (! myMesh1.ReadFromPly2Format(inFile))
		{
			cout << "\nError reading PLY2 file " << filename << endl;
		}
		cout << "\nsuccess reading PLY2 file " << filename << endl;

		glui->sync_live();
	}
	else
	{
		if (! myMesh1.ReadFromVRML(inFile))
		{
			cout << "\nerror reading VRML file " << filename << endl;
			inFile.close();
			return false;
		}
		else 
		{
			cout << endl << "\nsuccess reading VRML file " << filename << endl;
			
			glui->sync_live();
		}
	}
	
	
	inFile.close();
	
	cout << "The model (before cleanup):\n";
	cout << "\tnumber of vertices: " << myMesh1.GetNumberOfVertices() << endl;
	cout << "\tnumber of triangles: " << myMesh1.GetNumberOfTriangles() << endl;
	
	
	/******************************************************************************/
	// Getting ready to render

//	myMesh1.ReverseAllNormals();
	
	myMesh1.CleanDegenerateTripplets();

	
	// translate the mesh to the center of mass
	Point3D cent = myMesh1.GetCenterOfMass();
	myMesh1.Translate(-1.0 * cent);
	
	// scale mesh according to view
	Box bb = myMesh1.GetBoundingBox();
	real max_edge = bb.GetMaxEdgeLength();
	
	myMesh1.Scale(1.0/max_edge);
	
	myMesh1.SetMesh2Render();
	
	bb = myMesh1.GetBoundingBox();
	cout << bb.GetFront() << " " << bb.GetBack() << " " << bb.GetRight() << " " << bb.GetLeft() << " " << bb.GetTop() << " " << bb.GetBottom() << endl; 
	
	live_circle_radius = myMesh1.GetAverageEdgeLength() / 10.0f;
	circle_radius_spinner->set_float_limits(0.0f, 100*live_circle_radius);

	live_min_edge_length = myMesh1.GetMinEdgeLength();
	glui->sync_live();
	
	// For arcball
	Box3 bb1(bb.GetLeft(), bb.GetBack(), bb.GetBottom(),
		bb.GetRight(), bb.GetFront(), bb.GetTop());
	
	_view.init_exterior_view(bb1);
	Vector3 towards(0,0,-1);
	Vector3 up(0, 1, 0);
	_view.camera().reset(_origin - Vector3(0,0,-1.5), towards, up, M_PI/6.0, M_PI/6.0/xy_aspect, 0.1, 14.0); 
	_scale_factor = bb.GetMaxEdgeLength() * 0.01;

	mesh_bb = bb;
	
	control_cb(RESET_VIEW_ID);
	/******************************************************************************/


	is_reading_mesh = false;
	return true;
}


void myGlutVisible(int state)
{
	if (state == GLUT_VISIBLE)
		glutIdleFunc(myGlutIdle);
	else
		glutIdleFunc(NULL);
}


void SelectVertexUnderCursor(int x, int y)
{
    static const int MAXSELECT=1000;
    GLuint selectBuf[MAXSELECT];
    GLint vp[4];

    glSelectBuffer(MAXSELECT, selectBuf);

    (void)glRenderMode(GL_SELECT);
    glInitNames();
    glPushName(12345678);

    selected_vertex = -1;

    glGetIntegerv(GL_VIEWPORT, vp);

    {
        // open gl 0,0 is bottom left, while windows is top left?
        int window_height = glutGet(GLUT_WINDOW_HEIGHT);
        y = window_height - y;
        glMatrixMode(GL_PROJECTION);
        GLfloat m[16];
        glGetFloatv(GL_PROJECTION_MATRIX, m);
        glLoadIdentity();
        gluPickMatrix(x, y, 8, 8, vp);
        glMultMatrixf(m);
        glMatrixMode(GL_MODELVIEW );
    }

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    //
    // Render the triangles
    //
    unsigned N = myMesh.GetNumberOfVertices();

	std::vector<Point3D>::const_iterator v;
	std::vector<Point3D>::const_iterator nv;

	v = myMesh.GetConstVertexArray();
	nv = myMesh.GetConstNormalsArray();

	for (unsigned i = 0; i < N; i++) 
	{
        glLoadName(i);

		glBegin(GL_POINTS);
//		glNormal3fv(nv[i].GetCoordArray());
		glVertex3fv(v[i].GetCoordArray());
        glEnd();
	}


    //
    // End of rendering
    //

    int hits = glRenderMode(GL_RENDER); 

    if (hits < 0) 
    {
    	cout << "Hit buffer is full" << endl;
    }
    else if (hits == 0) 
	{ 
		selected_vertex = -1; 
		cout << "Disselected" << endl;
		return; 
	}
    else
    {
        GLuint names, *ptr;
        ptr = selectBuf;

        int selected_id;
        double closest=1e8;

        for (int i = 0; i < hits; i++) 
        {	/*  for each hit  */
            names = *ptr; ++ptr;
            double z1 = (double) *ptr/0x7fffffff; ++ptr;
            double z2 = (double) *ptr/0x7fffffff; ++ptr;
	    (void) z2;
            if (z1 < closest)
            {
              closest = z1;
              selected_id = *ptr;
            }
            ptr += names;
        }

        selected_vertex = selected_id;
        cout << "Selected vertex: " << selected_vertex << endl;
    }
}


bool RenderMeshToPPM(const char *filename) 
{
    int dims[4];

	glPixelStorei(GL_PACK_ALIGNMENT, 1);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glPixelStorei(GL_UNPACK_ROW_LENGTH , 0); 

    glGetIntegerv(GL_VIEWPORT , dims);
    BYTE* frame = new BYTE[dims[2]*dims[3]*3];

    glReadPixels(dims[0], dims[1], dims[2], dims[3], GL_RGB,
				 GL_UNSIGNED_BYTE, frame);


    FILE* f = fopen(filename,"wb");
    
	if (f == 0) 
		return false;
	
	fprintf(f, "P6\n%d %d 255\n", dims[2], dims[3]);
	fwrite(frame, 1, dims[2]*dims[3]*3, f);
    fclose(f);

	return true;
}



char * RemoveExtension(char *filename) {
	char * ret = NULL;
	char *ep = strstr(filename, ".");

	if (ep) {
		int length = ep - filename + 1;
	
		ret = new char[length];
		memcpy(ret, filename, length*sizeof(char));
		ret[length - 1] = '\0';
	}
	
	return ret;
}

void openMesh() {
	OPENFILENAME ofn;       // common dialog box structure
	char szFile[260];       // buffer for file name
	//		HWND hwnd;              // owner window
	//		HANDLE hf;              // file handle

	// Initialize OPENFILENAME
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = NULL;
	ofn.lpstrFile = szFile;
	//
	// Set lpstrFile[0] to '\0' so that GetOpenFileName does not 
	// use the contents of szFile to initialize itself.
	//
	ofn.lpstrFile[0] = '\0';
	ofn.nMaxFile = sizeof(szFile);
	                   //Name of file type\0Extension\0...
	ofn.lpstrFilter = "VRML 2.0 (.wrl)\0*.wrl\0PLY2 (.ply2)\0*.ply2\0Simple format (.smpl)\0*.smpl\0All files (*.*)\0*.*\0"; 
	ofn.nFilterIndex = 0;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	if (GetOpenFileName(&ofn)==TRUE) {
		std::string fileName = ofn.lpstrFile;
		
		//whatever you do with the file ...
		LoadMesh(myMesh, fileName.c_str());
		char filename[260];
		strcpy(filename, fileName.c_str());
		inFileEdit->set_text(filename);
	}
}
